@@ -14,9 +14,10 @@ intersphinx_mapping = {'python': ('http://docs.python.org/2.7', None)}
 source_suffix = '.rst'
 master_doc = 'index'
 project = 'mutagen'
-copyright = u'2013, Joe Wreschnig, Michael Urman, Lukáš Lalinský, ' \
-            u'Christoph Reiter & others'
+copyright = u'2014, Joe Wreschnig, Michael Urman, Lukáš Lalinský, ' \
+            u'Christoph Reiter, Ben Ockmore & others'
 version = mutagen.version_string
 release = mutagen.version_string
 exclude_patterns = ['_build']
 bug_url_template = "http://bitbucket.org/lazka/mutagen/issue/%s"
+pr_url_template = "http://bitbucket.org/lazka/mutagen/pull-request/%s"
