@@ -1,5 +1,8 @@
-# Copyright 2006-2007 Lukas Lalinsky
-# Copyright 2005-2006 Joe Wreschnig
+# -*- coding: utf-8 -*-
+
+# Copyright (C) 2005-2006  Joe Wreschnig
+# Copyright (C) 2006-2007  Lukas Lalinsky
+
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -12,9 +15,10 @@ __all__ = ["ASF", "Open"]
 import sys
 import struct
 from mutagen import FileType, Metadata, StreamInfo
-from mutagen._util import insert_bytes, delete_bytes, DictMixin, \
-    total_ordering, MutagenError
-from ._compat import swap_to_string, text_type, PY2, string_types, reraise
+from mutagen._util import (insert_bytes, delete_bytes, DictMixin,
+                           total_ordering, MutagenError)
+from ._compat import swap_to_string, text_type, PY2, string_types, reraise, \
+    xrange, long_, PY3
 
 
 class error(IOError, MutagenError):
@@ -48,7 +52,7 @@ class ASFTags(list, DictMixin, Metadata):
     """Dictionary containing ASF attributes."""
 
     def pprint(self):
-        return "\n".join(["%s=%s" % (k, v) for k, v in self])
+        return "\n".join("%s=%s" % (k, v) for k, v in self)
 
     def __getitem__(self, key):
         """A list of values for the key.
@@ -57,6 +61,11 @@ class ASFTags(list, DictMixin, Metadata):
         work.
 
         """
+
+        # PY3 only
+        if isinstance(key, slice):
+            return list.__getitem__(self, key)
+
         values = [value for (k, value) in self if k == key]
         if not values:
             raise KeyError(key)
@@ -65,7 +74,12 @@ class ASFTags(list, DictMixin, Metadata):
 
     def __delitem__(self, key):
         """Delete all values associated with the key."""
-        to_delete = list(filter(lambda x: x[0] == key, self))
+
+        # PY3 only
+        if isinstance(key, slice):
+            return list.__delitem__(self, key)
+
+        to_delete = [x for x in self if x[0] == key]
         if not to_delete:
             raise KeyError(key)
         else:
@@ -88,26 +102,37 @@ class ASFTags(list, DictMixin, Metadata):
         string.
 
         """
+
+        # PY3 only
+        if isinstance(key, slice):
+            return list.__setitem__(self, key, values)
+
         if not isinstance(values, list):
             values = [values]
-        try:
-            del(self[key])
-        except KeyError:
-            pass
+
+        to_append = []
         for value in values:
-            if key in _standard_attribute_names:
-                value = text_type(value)
-            elif not isinstance(value, ASFBaseAttribute):
+            if not isinstance(value, ASFBaseAttribute):
                 if isinstance(value, string_types):
-                    if PY2 or isinstance(value, text_type):
-                        value = ASFUnicodeAttribute(value)
+                    value = ASFUnicodeAttribute(value)
+                elif PY3 and isinstance(value, bytes):
+                    value = ASFByteArrayAttribute(value)
                 elif isinstance(value, bool):
                     value = ASFBoolAttribute(value)
                 elif isinstance(value, int):
                     value = ASFDWordAttribute(value)
-                elif isinstance(value, long):
+                elif isinstance(value, long_):
                     value = ASFQWordAttribute(value)
-            self.append((key, value))
+                else:
+                    raise TypeError("Invalid type %r" % type(value))
+            to_append.append((key, value))
+
+        try:
+            del(self[key])
+        except KeyError:
+            pass
+
+        self.extend(to_append)
 
     def keys(self):
         """Return all keys in the comment."""
@@ -248,7 +273,10 @@ class ASFByteArrayAttribute(ASFBaseAttribute):
         return len(self.value)
 
     def __bytes__(self):
-        return "[binary data (%s bytes)]" % len(self.value)
+        return self.value
+
+    def __str__(self):
+        return "[binary data (%d bytes)]" % len(self.value)
 
     def __eq__(self, other):
         return self.value == other
@@ -289,6 +317,9 @@ class ASFBoolAttribute(ASFBaseAttribute):
     def __bytes__(self):
         return text_type(self.value).encode('utf-8')
 
+    def __str__(self):
+        return text_type(self.value)
+
     def __eq__(self, other):
         return bool(self.value) == other
 
@@ -323,7 +354,10 @@ class ASFDWordAttribute(ASFBaseAttribute):
         return self.value
 
     def __bytes__(self):
-        return self.value
+        return text_type(self.value).encode('utf-8')
+
+    def __str__(self):
+        return text_type(self.value)
 
     def __eq__(self, other):
         return int(self.value) == other
@@ -359,7 +393,10 @@ class ASFQWordAttribute(ASFBaseAttribute):
         return self.value
 
     def __bytes__(self):
-        return self.value
+        return text_type(self.value).encode('utf-8')
+
+    def __str__(self):
+        return text_type(self.value)
 
     def __eq__(self, other):
         return int(self.value) == other
@@ -395,7 +432,10 @@ class ASFWordAttribute(ASFBaseAttribute):
         return self.value
 
     def __bytes__(self):
-        return self.value
+        return text_type(self.value).encode('utf-8')
+
+    def __str__(self):
+        return text_type(self.value)
 
     def __eq__(self, other):
         return int(self.value) == other
@@ -431,6 +471,9 @@ class ASFGUIDAttribute(ASFBaseAttribute):
     def __bytes__(self):
         return self.value
 
+    def __str__(self):
+        return repr(self.value)
+
     def __eq__(self, other):
         return self.value == other
 
@@ -450,10 +493,12 @@ GUID = ASFGUIDAttribute.TYPE
 
 
 def ASFValue(value, kind, **kwargs):
-    for t, c in _attribute_types.items():
-        if kind == t:
-            return c(value=value, **kwargs)
-    raise ValueError("Unknown value type")
+    try:
+        attr_type = _attribute_types[kind]
+    except KeyError:
+        raise ValueError("Unknown value type %r" % kind)
+    else:
+        return attr_type(value=value, **kwargs)
 
 
 _attribute_types = {
@@ -467,15 +512,6 @@ _attribute_types = {
 }
 
 
-_standard_attribute_names = [
-    "Title",
-    "Author",
-    "Copyright",
-    "Description",
-    "Rating"
-]
-
-
 class BaseObject(object):
     """Base ASF object."""
     GUID = None
@@ -504,6 +540,14 @@ class ContentDescriptionObject(BaseObject):
     """Content description."""
     GUID = b"\x33\x26\xB2\x75\x8E\x66\xCF\x11\xA6\xD9\x00\xAA\x00\x62\xCE\x6C"
 
+    NAMES = [
+        u"Title",
+        u"Author",
+        u"Copyright",
+        u"Description",
+        u"Rating",
+    ]
+
     def parse(self, asf, data, fileobj, size):
         super(ContentDescriptionObject, self).parse(asf, data, fileobj, size)
         asf.content_description_obj = self
@@ -513,29 +557,25 @@ class ContentDescriptionObject(BaseObject):
         for length in lengths:
             end = pos + length
             if length > 0:
-                texts.append(data[pos:end].decode("utf-16-le").strip("\x00"))
+                texts.append(data[pos:end].decode("utf-16-le").strip(u"\x00"))
             else:
                 texts.append(None)
             pos = end
-        title, author, copyright, desc, rating = texts
-        for key, value in dict(
-            Title=title,
-            Author=author,
-            Copyright=copyright,
-            Description=desc,
-            Rating=rating
-        ).items():
+
+        for key, value in zip(self.NAMES, texts):
             if value is not None:
-                asf.tags[key] = value
+                value = ASFUnicodeAttribute(value=value)
+                asf._tags.setdefault(self.GUID, []).append((key, value))
 
     def render(self, asf):
         def render_text(name):
-            value = asf.tags.get(name, [])
-            if value:
-                return value[0].encode("utf-16-le") + b"\x00\x00"
+            value = asf.to_content_description.get(name)
+            if value is not None:
+                return text_type(value).encode("utf-16-le") + b"\x00\x00"
             else:
                 return b""
-        texts = list(map(render_text, _standard_attribute_names))
+
+        texts = [render_text(x) for x in self.NAMES]
         data = struct.pack("<HHHHH", *map(len, texts)) + b"".join(texts)
         return self.GUID + struct.pack("<Q", 24 + len(data)) + data
 
@@ -550,7 +590,7 @@ class ExtendedContentDescriptionObject(BaseObject):
         asf.extended_content_description_obj = self
         num_attributes, = struct.unpack("<H", data[0:2])
         pos = 2
-        for i in range(num_attributes):
+        for i in xrange(num_attributes):
             name_length, = struct.unpack("<H", data[pos:pos + 2])
             pos += 2
             name = data[pos:pos + name_length]
@@ -561,11 +601,11 @@ class ExtendedContentDescriptionObject(BaseObject):
             value = data[pos:pos + value_length]
             pos += value_length
             attr = _attribute_types[value_type](data=value)
-            asf.tags.append((name, attr))
+            asf._tags.setdefault(self.GUID, []).append((name, attr))
 
     def render(self, asf):
         attrs = asf.to_extended_content_description.items()
-        data = b"".join([attr.render(name) for (name, attr) in attrs])
+        data = b"".join(attr.render(name) for (name, attr) in attrs)
         data = struct.pack("<QH", 26 + len(data), len(attrs)) + data
         return self.GUID + data
 
@@ -577,7 +617,7 @@ class FilePropertiesObject(BaseObject):
     def parse(self, asf, data, fileobj, size):
         super(FilePropertiesObject, self).parse(asf, data, fileobj, size)
         length, _, preroll = struct.unpack("<QQQ", data[40:64])
-        asf.info.length = length / 10000000.0 - preroll / 1000.0
+        asf.info.length = (length / 10000000.0) - (preroll / 1000.0)
 
 
 class StreamPropertiesObject(BaseObject):
@@ -615,7 +655,7 @@ class HeaderExtensionObject(BaseObject):
             datapos += size
 
     def render(self, asf):
-        data = b"".join([obj.render(asf) for obj in self.objects])
+        data = b"".join(obj.render(asf) for obj in self.objects)
         return (self.GUID + struct.pack("<Q", 24 + 16 + 6 + len(data)) +
                 b"\x11\xD2\xD3\xAB\xBA\xA9\xcf\x11" +
                 b"\x8E\xE6\x00\xC0\x0C\x20\x53\x65" +
@@ -631,7 +671,7 @@ class MetadataObject(BaseObject):
         asf.metadata_obj = self
         num_attributes, = struct.unpack("<H", data[0:2])
         pos = 2
-        for i in range(num_attributes):
+        for i in xrange(num_attributes):
             (reserved, stream, name_length, value_type,
              value_length) = struct.unpack("<HHHHI", data[pos:pos + 12])
             pos += 12
@@ -644,7 +684,7 @@ class MetadataObject(BaseObject):
             if value_type == 2:
                 args['dword'] = False
             attr = _attribute_types[value_type](**args)
-            asf.tags.append((name, attr))
+            asf._tags.setdefault(self.GUID, []).append((name, attr))
 
     def render(self, asf):
         attrs = asf.to_metadata.items()
@@ -662,7 +702,7 @@ class MetadataLibraryObject(BaseObject):
         asf.metadata_library_obj = self
         num_attributes, = struct.unpack("<H", data[0:2])
         pos = 2
-        for i in range(num_attributes):
+        for i in xrange(num_attributes):
             (language, stream, name_length, value_type,
              value_length) = struct.unpack("<HHHHI", data[pos:pos + 12])
             pos += 12
@@ -675,7 +715,7 @@ class MetadataLibraryObject(BaseObject):
             if value_type == 2:
                 args['dword'] = False
             attr = _attribute_types[value_type](**args)
-            asf.tags.append((name, attr))
+            asf._tags.setdefault(self.GUID, []).append((name, attr))
 
     def render(self, asf):
         attrs = asf.to_metadata_library
@@ -703,8 +743,7 @@ class ASF(FileType):
 
     def load(self, filename):
         self.filename = filename
-        fileobj = open(filename, "rb")
-        try:
+        with open(filename, "rb") as fileobj:
             self.size = 0
             self.size1 = 0
             self.size2 = 0
@@ -714,27 +753,34 @@ class ASF(FileType):
             self.info = ASFInfo()
             self.tags = ASFTags()
             self.__read_file(fileobj)
-        finally:
-            fileobj.close()
 
     def save(self):
         # Move attributes to the right objects
+        self.to_content_description = {}
         self.to_extended_content_description = {}
         self.to_metadata = {}
         self.to_metadata_library = []
         for name, value in self.tags:
-            if name in _standard_attribute_names:
-                continue
             library_only = (value.data_size() > 0xFFFF or value.TYPE == GUID)
-            if (value.language is None and value.stream is None and
-                    name not in self.to_extended_content_description and
-                    not library_only):
-                self.to_extended_content_description[name] = value
-            elif (value.language is None and value.stream is not None and
-                  name not in self.to_metadata and not library_only):
-                self.to_metadata[name] = value
-            else:
+            can_cont_desc = value.TYPE == UNICODE
+
+            if library_only or value.language is not None:
                 self.to_metadata_library.append((name, value))
+            elif value.stream is not None:
+                if name not in self.to_metadata:
+                    self.to_metadata[name] = value
+                else:
+                    self.to_metadata_library.append((name, value))
+            elif name in ContentDescriptionObject.NAMES:
+                if name not in self.to_content_description and can_cont_desc:
+                    self.to_content_description[name] = value
+                else:
+                    self.to_metadata_library.append((name, value))
+            else:
+                if name not in self.to_extended_content_description:
+                    self.to_extended_content_description[name] = value
+                else:
+                    self.to_metadata_library.append((name, value))
 
         # Add missing objects
         if not self.content_description_obj:
@@ -764,8 +810,7 @@ class ASF(FileType):
                 struct.pack("<QL", len(data) + 30, len(self.objects)) +
                 b"\x01\x02" + data)
 
-        fileobj = open(self.filename, "rb+")
-        try:
+        with open(self.filename, "rb+") as fileobj:
             size = len(data)
             if size > self.size:
                 insert_bytes(fileobj, size - self.size, self.size)
@@ -773,8 +818,6 @@ class ASF(FileType):
                 delete_bytes(fileobj, self.size - size, 0)
             fileobj.seek(0)
             fileobj.write(data)
-        finally:
-            fileobj.close()
 
         self.size = size
         self.num_objects = len(self.objects)
@@ -792,9 +835,16 @@ class ASF(FileType):
 
         self.size, self.num_objects = struct.unpack("<QL", header[16:28])
         self.objects = []
-        for i in range(self.num_objects):
+        self._tags = {}
+        for i in xrange(self.num_objects):
             self.__read_object(fileobj)
 
+        for guid in [ContentDescriptionObject.GUID,
+                ExtendedContentDescriptionObject.GUID, MetadataObject.GUID,
+                MetadataLibraryObject.GUID]:
+            self.tags.extend(self._tags.pop(guid, []))
+        assert not self._tags
+
     def __read_object(self, fileobj):
         guid, size = struct.unpack("<16sQ", fileobj.read(24))
         if guid in _object_types:
