@@ -1,5 +1,6 @@
-# FLAC comment support for Mutagen
-# Copyright 2005 Joe Wreschnig
+# -*- coding: utf-8 -*-
+
+# Copyright (C) 2005  Joe Wreschnig
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of version 2 of the GNU General Public License as
@@ -82,6 +83,13 @@ class MetadataBlock(object):
     """
 
     _distrust_size = False
+    """For block types setting this, we don't trust the size field and
+    use the size of the content instead."""
+
+    _invalid_overflow_size = -1
+    """In case the real size was bigger than what is representable by the
+    24 bit size field, we save the wrong specified size here. This can
+    only be set if _distrust_size is True"""
 
     def __init__(self, data):
         """Parse the given data string or file-like as a metadata block.
@@ -105,15 +113,26 @@ class MetadataBlock(object):
     @staticmethod
     def writeblocks(blocks):
         """Render metadata block as a byte string."""
+
         data = []
-        codes = [[block.code, block.write()] for block in blocks]
-        codes[-1][0] |= 128
-        for code, datum in codes:
-            byte = chr_(code)
-            if len(datum) > 2 ** 24:
-                raise error("block is too long to write")
-            length = struct.pack(">I", len(datum))[-3:]
-            data.append(byte + length + datum)
+        for i, block in enumerate(blocks):
+            is_last = (i == len(blocks) - 1)
+            code = (block.code | 128) if is_last else block.code
+            datum = block.write()
+            size = len(datum)
+            if size > 2 ** 24:
+                if block._distrust_size and block._invalid_overflow_size != -1:
+                    # The original size of this block was (1) wrong and (2)
+                    # the real size doesn't allow us to save the file
+                    # according to the spec (too big for 24 bit uint). Instead
+                    # simply write back the original wrong size.. at least
+                    # we don't make the file more "broken" as it is.
+                    size = block._invalid_overflow_size
+                else:
+                    raise error("block is too long to write")
+            assert not size > 2 ** 24
+            length = struct.pack(">I", size)[-3:]
+            data.extend([chr_(code), length, datum])
         return b"".join(data)
 
     @staticmethod
@@ -129,7 +148,7 @@ class MetadataBlock(object):
             blocks.remove(p)
         # total padding size is the sum of padding sizes plus 4 bytes
         # per removed header.
-        size = sum([padding.length for padding in paddings])
+        size = sum(padding.length for padding in paddings)
         padding = Padding()
         padding.length = size + 4 * (len(paddings) - 1)
         blocks.append(padding)
@@ -380,10 +399,10 @@ class CueSheetTrack(object):
     __hash__ = object.__hash__
 
     def __repr__(self):
-        return ("<%s number=%r, offset=%d, isrc=%r, type=%r, "
-                "pre_emphasis=%r, indexes=%r)>") % (
-                    type(self).__name__, self.track_number, self.start_offset,
-                    self.isrc, self.type, self.pre_emphasis, self.indexes)
+        return (("<%s number=%r, offset=%d, isrc=%r, type=%r, "
+                "pre_emphasis=%r, indexes=%r)>") %
+                (type(self).__name__, self.track_number, self.start_offset,
+                 self.isrc, self.type, self.pre_emphasis, self.indexes))
 
 
 class CueSheet(MetadataBlock):
@@ -482,10 +501,10 @@ class CueSheet(MetadataBlock):
         return f.getvalue()
 
     def __repr__(self):
-        return ("<%s media_catalog_number=%r, lead_in=%r, compact_disc=%r, "
-                "tracks=%r>") % (
-                    type(self).__name__, self.media_catalog_number,
-                    self.lead_in_samples, self.compact_disc, self.tracks)
+        return (("<%s media_catalog_number=%r, lead_in=%r, compact_disc=%r, "
+                 "tracks=%r>") %
+                (type(self).__name__, self.media_catalog_number,
+                 self.lead_in_samples, self.compact_disc, self.tracks))
 
 
 class Picture(MetadataBlock):
@@ -502,6 +521,21 @@ class Picture(MetadataBlock):
     * colors -- number of colors for indexed palettes (like GIF),
       0 for non-indexed
     * data -- picture data
+
+    To create a picture from file (in order to add to a FLAC file),
+    instantiate this object without passing anything to the constructor and
+    then set the properties manually::
+
+        p = Picture()
+
+        with open("Folder.jpg", "rb") as f:
+            pic.data = f.read()
+
+        pic.type = id3.PictureType.COVER_FRONT
+        pic.mime = u"image/jpeg"
+        pic.width = 500
+        pic.height = 500
+        pic.depth = 16 # color depth
     """
 
     code = 6
@@ -642,7 +676,11 @@ class FLAC(mutagen.FileType):
             # http://code.google.com/p/mutagen/issues/detail?id=52
             # ..same for the Picture block:
             # http://code.google.com/p/mutagen/issues/detail?id=106
+            start = fileobj.tell()
             block = block_type(fileobj)
+            real_size = fileobj.tell() - start
+            if real_size > 2 ** 24:
+                block._invalid_overflow_size = size
         else:
             data = fileobj.read(size)
             block = block_type(data)
