@@ -1,4 +1,5 @@
-# id3 support for mutagen
+# -*- coding: utf-8 -*-
+
 # Copyright (C) 2005  Michael Urman
 #               2006  Lukas Lalinsky
 #               2013  Christoph Reiter
@@ -58,6 +59,109 @@ class ID3v1SaveOptions(object):
     """ID3v1 tags will be created and/or updated"""
 
 
+def _fullread(fileobj, size):
+    """Read a certain number of bytes from the source file.
+
+    Raises ValueError on invalid size input or EOFError/IOError.
+    """
+
+    if size < 0:
+        raise ValueError('Requested bytes (%s) less than zero' % size)
+    data = fileobj.read(size)
+    if len(data) != size:
+        raise EOFError("Not enough data to read")
+    return data
+
+
+class ID3Header(object):
+
+    _V24 = (2, 4, 0)
+    _V23 = (2, 3, 0)
+    _V22 = (2, 2, 0)
+    _V11 = (1, 1)
+
+    f_unsynch = property(lambda s: bool(s._flags & 0x80))
+    f_extended = property(lambda s: bool(s._flags & 0x40))
+    f_experimental = property(lambda s: bool(s._flags & 0x20))
+    f_footer = property(lambda s: bool(s._flags & 0x10))
+
+    def __init__(self, fileobj=None):
+        """Raises ID3NoHeaderError, ID3UnsupportedVersionError or error"""
+
+        if fileobj is None:
+            # for testing
+            self._flags = 0
+            return
+
+        fn = getattr(fileobj, "name", "<unknown>")
+        try:
+            data = _fullread(fileobj, 10)
+        except EOFError:
+            raise ID3NoHeaderError("%s: too small" % fn)
+
+        id3, vmaj, vrev, flags, size = unpack('>3sBBB4s', data)
+        self._flags = flags
+        self.size = BitPaddedInt(size) + 10
+        self.version = (2, vmaj, vrev)
+
+        if id3 != b'ID3':
+            raise ID3NoHeaderError("%r doesn't start with an ID3 tag" % fn)
+
+        if vmaj not in [2, 3, 4]:
+            raise ID3UnsupportedVersionError("%r ID3v2.%d not supported"
+                                             % (fn, vmaj))
+
+        if not BitPaddedInt.has_valid_padding(size):
+            raise error("Header size not synchsafe")
+
+        if (self.version >= self._V24) and (flags & 0x0f):
+            raise error(
+                "%r has invalid flags %#02x" % (fn, flags))
+        elif (self._V23 <= self.version < self._V24) and (flags & 0x1f):
+            raise error(
+                "%r has invalid flags %#02x" % (fn, flags))
+
+        if self.f_extended:
+            try:
+                extsize_data = _fullread(fileobj, 4)
+            except EOFError:
+                raise error("%s: too small" % fn)
+
+            if PY3:
+                frame_id = extsize_data.decode("ascii", "replace")
+            else:
+                frame_id = extsize_data
+
+            if frame_id in Frames:
+                # Some tagger sets the extended header flag but
+                # doesn't write an extended header; in this case, the
+                # ID3 data follows immediately. Since no extended
+                # header is going to be long enough to actually match
+                # a frame, and if it's *not* a frame we're going to be
+                # completely lost anyway, this seems to be the most
+                # correct check.
+                # http://code.google.com/p/quodlibet/issues/detail?id=126
+                self._flags ^= 0x40
+                extsize = 0
+                fileobj.seek(-4, 1)
+            elif self.version >= self._V24:
+                # "Where the 'Extended header size' is the size of the whole
+                # extended header, stored as a 32 bit synchsafe integer."
+                extsize = BitPaddedInt(extsize_data) - 4
+                if not BitPaddedInt.has_valid_padding(extsize_data):
+                    raise error(
+                        "Extended header size not synchsafe")
+            else:
+                # "Where the 'Extended header size', currently 6 or 10 bytes,
+                # excludes itself."
+                extsize = unpack('>L', extsize_data)[0]
+
+            try:
+                self._extdata = _fullread(fileobj, extsize)
+            except EOFError:
+                raise error("%s: too small" % fn)
+
+
 class ID3(DictProxy, mutagen.Metadata):
     """A file with an ID3v2 tag.
 
@@ -68,39 +172,53 @@ class ID3(DictProxy, mutagen.Metadata):
     * size -- the total size of the ID3 tag, including the header
     """
 
+    __module__ = "mutagen.id3"
+
     PEDANTIC = True
-    version = (2, 4, 0)
+    """Deprecated. Doesn't have any effect"""
 
     filename = None
-    size = 0
-    __flags = 0
-    __readbytes = 0
-    __crc = None
-    __unknown_version = None
-
-    _V24 = (2, 4, 0)
-    _V23 = (2, 3, 0)
-    _V22 = (2, 2, 0)
-    _V11 = (1, 1)
 
     def __init__(self, *args, **kwargs):
         self.unknown_frames = []
+        self.__unknown_version = None
+        self._header = None
+        self._version = (2, 4, 0)
         super(ID3, self).__init__(*args, **kwargs)
 
-    def __fullread(self, size):
-        try:
-            if size < 0:
-                raise ValueError('Requested bytes (%s) less than zero' % size)
-            if size > self.__filesize:
-                raise EOFError('Requested %#x of %#x (%s)' % (
-                    int(size), int(self.__filesize), self.filename))
-        except AttributeError:
-            pass
-        data = self._fileobj.read(size)
-        if len(data) != size:
-            raise EOFError
-        self.__readbytes += size
-        return data
+    @property
+    def version(self):
+        """ID3 tag version as a tuple (of the loaded file)"""
+
+        if self._header is not None:
+            return self._header.version
+        return self._version
+
+    @version.setter
+    def version(self, value):
+        self._version = value
+
+    @property
+    def f_unsynch(self):
+        if self._header is not None:
+            return self._header.f_unsynch
+        return False
+
+    @property
+    def f_extended(self):
+        if self._header is not None:
+            return self._header.f_extended
+        return False
+
+    @property
+    def size(self):
+        if self._header is not None:
+            return self._header.size
+        return 0
+
+    def _pre_load_header(self, fileobj):
+        # XXX: for aiff to adjust the offset..
+        pass
 
     def load(self, filename, known_frames=None, translate=True, v2_version=4):
         """Load tags from a filename.
@@ -125,51 +243,49 @@ class ID3(DictProxy, mutagen.Metadata):
         if v2_version not in (3, 4):
             raise ValueError("Only 3 and 4 possible for v2_version")
 
-        from os.path import getsize
-
         self.filename = filename
+        self.unknown_frames = []
         self.__known_frames = known_frames
-        self._fileobj = open(filename, 'rb')
-        self.__filesize = getsize(filename)
-        try:
+        self._header = None
+
+        with open(filename, 'rb') as fileobj:
+            self._pre_load_header(fileobj)
+
             try:
-                self._load_header()
-            except EOFError:
-                self.size = 0
-                raise ID3NoHeaderError("%s: too small (%d bytes)" % (
-                    filename, self.__filesize))
+                self._header = ID3Header(fileobj)
             except (ID3NoHeaderError, ID3UnsupportedVersionError):
-                self.size = 0
-                frames, offset = _find_id3v1(self._fileobj)
+                frames, offset = _find_id3v1(fileobj)
                 if frames is None:
                     raise
 
-                self.version = self._V11
+                self.version = ID3Header._V11
                 for v in frames.values():
                     self.add(v)
             else:
                 frames = self.__known_frames
                 if frames is None:
-                    if self._V23 <= self.version:
+                    if self.version >= ID3Header._V23:
                         frames = Frames
-                    elif self._V22 <= self.version:
+                    elif self.version >= ID3Header._V22:
                         frames = Frames_2_2
-                data = self.__fullread(self.size - 10)
+
+                try:
+                    data = _fullread(fileobj, self.size - 10)
+                except (ValueError, EOFError, IOError) as e:
+                    raise error(e)
+
                 for frame in self.__read_frames(data, frames=frames):
                     if isinstance(frame, Frame):
                         self.add(frame)
                     else:
                         self.unknown_frames.append(frame)
                 self.__unknown_version = self.version[:2]
-        finally:
-            self._fileobj.close()
-            del self._fileobj
-            del self.__filesize
-            if translate:
-                if v2_version == 3:
-                    self.update_to_v23()
-                else:
-                    self.update_to_v24()
+
+        if translate:
+            if v2_version == 3:
+                self.update_to_v23()
+            else:
+                self.update_to_v24()
 
     def getall(self, key):
         """Return all frames with a given name (the list may be empty).
@@ -197,8 +313,9 @@ class ID3(DictProxy, mutagen.Metadata):
             del(self[key])
         else:
             key = key + ":"
-            for k in filter(lambda s: s.startswith(key), list(self.keys())):
-                del(self[k])
+            for k in list(self.keys()):
+                if k.startswith(key):
+                    del(self[k])
 
     def setall(self, key, values):
         """Delete frames of the given type and add frames in 'values'."""
@@ -218,8 +335,7 @@ class ID3(DictProxy, mutagen.Metadata):
 
             ``POPM=user@example.org=3 128/255``
         """
-        frames = list(map(Frame.pprint, self.values()))
-        frames.sort()
+        frames = sorted(Frame.pprint(s) for s in self.values())
         return "\n".join(frames)
 
     def loaded_frame(self, tag):
@@ -236,76 +352,21 @@ class ID3(DictProxy, mutagen.Metadata):
         """Add a frame to the tag."""
         return self.loaded_frame(frame)
 
-    def _load_header(self):
-        fn = self.filename
-        data = self.__fullread(10)
-        id3, vmaj, vrev, flags, size = unpack('>3sBBB4s', data)
-        self.__flags = flags
-        self.size = BitPaddedInt(size) + 10
-        self.version = (2, vmaj, vrev)
-
-        if id3 != b'ID3':
-            raise ID3NoHeaderError("%r doesn't start with an ID3 tag" % fn)
-        if vmaj not in [2, 3, 4]:
-            raise ID3UnsupportedVersionError("%r ID3v2.%d not supported"
-                                             % (fn, vmaj))
-
-        if self.PEDANTIC:
-            if not BitPaddedInt.has_valid_padding(size):
-                raise ValueError("Header size not synchsafe")
-
-            if self._V24 <= self.version and (flags & 0x0f):
-                raise ValueError("%r has invalid flags %#02x" % (fn, flags))
-            elif self._V23 <= self.version < self._V24 and (flags & 0x1f):
-                raise ValueError("%r has invalid flags %#02x" % (fn, flags))
-
-        if self.f_extended:
-            extsize = self.__fullread(4)
-            frame_id = extsize.decode("ascii", "replace") if PY3 else extsize
-            if frame_id in Frames:
-                # Some tagger sets the extended header flag but
-                # doesn't write an extended header; in this case, the
-                # ID3 data follows immediately. Since no extended
-                # header is going to be long enough to actually match
-                # a frame, and if it's *not* a frame we're going to be
-                # completely lost anyway, this seems to be the most
-                # correct check.
-                # http://code.google.com/p/quodlibet/issues/detail?id=126
-                self.__flags ^= 0x40
-                self.__extsize = 0
-                self._fileobj.seek(-4, 1)
-                self.__readbytes -= 4
-            elif self.version >= self._V24:
-                # "Where the 'Extended header size' is the size of the whole
-                # extended header, stored as a 32 bit synchsafe integer."
-                self.__extsize = BitPaddedInt(extsize) - 4
-                if self.PEDANTIC:
-                    if not BitPaddedInt.has_valid_padding(extsize):
-                        raise ValueError("Extended header size not synchsafe")
-            else:
-                # "Where the 'Extended header size', currently 6 or 10 bytes,
-                # excludes itself."
-                self.__extsize = unpack('>L', extsize)[0]
-            if self.__extsize:
-                self.__extdata = self.__fullread(self.__extsize)
-            else:
-                self.__extdata = b""
-
-    def __determine_bpi(self, data, frames):
-        if self.version < self._V24:
-            return int
-
-        return _determine_bpi(data, frames)
-
     def __read_frames(self, data, frames):
-        if self.version < self._V24 and self.f_unsynch:
+        assert self.version >= ID3Header._V22
+
+        if self.version < ID3Header._V24 and self.f_unsynch:
             try:
                 data = unsynch.decode(data)
             except ValueError:
                 pass
 
-        if self._V23 <= self.version:
-            bpi = self.__determine_bpi(data, frames)
+        if self.version >= ID3Header._V23:
+            if self.version < ID3Header._V24:
+                bpi = int
+            else:
+                bpi = _determine_bpi(data, frames)
+
             while data:
                 header = data[:10]
                 try:
@@ -339,13 +400,12 @@ class ID3(DictProxy, mutagen.Metadata):
                         yield header + framedata
                 else:
                     try:
-                        yield self.__load_framedata(tag, flags, framedata)
+                        yield tag._fromData(self._header, flags, framedata)
                     except NotImplementedError:
                         yield header + framedata
                     except ID3JunkFrameError:
                         pass
-
-        elif self._V22 <= self.version:
+        elif self.version >= ID3Header._V22:
             while data:
                 header = data[0:6]
                 try:
@@ -374,36 +434,27 @@ class ID3(DictProxy, mutagen.Metadata):
                         yield header + framedata
                 else:
                     try:
-                        yield self.__load_framedata(tag, 0, framedata)
-                    except NotImplementedError:
+                        yield tag._fromData(self._header, 0, framedata)
+                    except (ID3EncryptionUnsupportedError,
+                            NotImplementedError):
                         yield header + framedata
                     except ID3JunkFrameError:
                         pass
 
-    def __load_framedata(self, tag, flags, framedata):
-        return tag.fromData(self, flags, framedata)
-
-    f_unsynch = property(lambda s: bool(s.__flags & 0x80))
-    f_extended = property(lambda s: bool(s.__flags & 0x40))
-    f_experimental = property(lambda s: bool(s.__flags & 0x20))
-    f_footer = property(lambda s: bool(s.__flags & 0x10))
-
-    # f_crc = property(lambda s: bool(s.__extflags & 0x8000))
-
     def _prepare_framedata(self, v2_version, v23_sep):
         if v2_version == 3:
-            version = self._V23
+            version = ID3Header._V23
         elif v2_version == 4:
-            version = self._V24
+            version = ID3Header._V24
         else:
             raise ValueError("Only 3 or 4 allowed for v2_version")
 
         # Sort frames by 'importance'
         order = ["TIT2", "TPE1", "TRCK", "TALB", "TPOS", "TDRC", "TCON"]
-        order = dict(zip(order, range(len(order))))
+        order = dict((b, a) for a, b in enumerate(order))
         last = len(order)
-        frames = self.items()
-        frames.sort(key=lambda a: (order.get(a[0][:4], last), a[0]))
+        frames = sorted(self.items(),
+                        key=lambda a: (order.get(a[0][:4], last), a[0]))
 
         framedata = [self.__save_frame(frame, version=version, v23_sep=v23_sep)
                      for (key, frame) in frames]
@@ -411,8 +462,8 @@ class ID3(DictProxy, mutagen.Metadata):
         # only write unknown frames if they were loaded from the version
         # we are saving with or upgraded to it
         if self.__unknown_version == version[:2]:
-            framedata.extend([data for data in self.unknown_frames
-                              if len(data) > 10])
+            framedata.extend(data for data in self.unknown_frames
+                             if len(data) > 10)
 
         return b''.join(framedata)
 
@@ -439,21 +490,24 @@ class ID3(DictProxy, mutagen.Metadata):
     def save(self, filename=None, v1=1, v2_version=4, v23_sep='/'):
         """Save changes to a file.
 
-        If no filename is given, the one most recently loaded is used.
-
-        Keyword arguments:
-        v1 -- if 0, ID3v1 tags will be removed
-              if 1, ID3v1 tags will be updated but not added
-              if 2, ID3v1 tags will be created and/or updated
-        v2 -- version of ID3v2 tags (3 or 4).
+        Args:
+            filename:
+                Filename to save the tag to. If no filename is given,
+                the one most recently loaded is used.
+            v1 (ID3v1SaveOptions):
+                if 0, ID3v1 tags will be removed.
+                if 1, ID3v1 tags will be updated but not added.
+                if 2, ID3v1 tags will be created and/or updated
+            v2 (int):
+                version of ID3v2 tags (3 or 4).
+            v23_sep (str):
+                the separator used to join multiple text values
+                if v2_version == 3. Defaults to '/' but if it's None
+                will be the ID3v2v2.4 null separator.
 
         By default Mutagen saves ID3v2.4 tags. If you want to save ID3v2.3
         tags, you must call method update_to_v23 before saving the file.
 
-        v23_sep -- the separator used to join multiple text values
-                   if v2_version == 3. Defaults to '/' but if it's None
-                   will be the ID3v2v2.4 null separator.
-
         The lack of a way to update only an ID3v1 tag is intentional.
         """
 
@@ -523,13 +577,14 @@ class ID3(DictProxy, mutagen.Metadata):
         delete(filename, delete_v1, delete_v2)
         self.clear()
 
-    def __save_frame(self, frame, name=None, version=_V24, v23_sep=None):
+    def __save_frame(self, frame, name=None, version=ID3Header._V24,
+                     v23_sep=None):
         flags = 0
-        if self.PEDANTIC and isinstance(frame, TextFrame):
+        if isinstance(frame, TextFrame):
             if len(str(frame)) == 0:
                 return b''
 
-        if version == self._V23:
+        if version == ID3Header._V23:
             framev23 = frame._get_v23_frame(sep=v23_sep)
             framedata = framev23._writeData()
         else:
@@ -544,9 +599,9 @@ class ID3(DictProxy, mutagen.Metadata):
             # flags |= Frame.FLAG24_COMPRESS | Frame.FLAG24_DATALEN
             pass
 
-        if version == self._V24:
+        if version == ID3Header._V24:
             bits = 7
-        elif version == self._V23:
+        elif version == ID3Header._V23:
             bits = 8
         else:
             raise ValueError
@@ -600,8 +655,13 @@ class ID3(DictProxy, mutagen.Metadata):
             for frame in self.unknown_frames:
                 try:
                     name, size, flags = unpack('>4sLH', frame[:10])
-                    frame = BinaryFrame.fromData(self, flags, frame[10:])
-                except (struct.error, error):
+                except struct.error:
+                    continue
+
+                try:
+                    frame = BinaryFrame._fromData(
+                        self._header, flags, frame[10:])
+                except (error, NotImplementedError):
                     continue
 
                 converted.append(self.__save_frame(frame, name=name))
@@ -721,26 +781,26 @@ def delete(filename, delete_v1=True, delete_v2=True):
     * delete_v2 -- delete any ID3v2 tag
     """
 
-    f = open(filename, 'rb+')
+    with open(filename, 'rb+') as f:
 
-    if delete_v1:
-        tag, offset = _find_id3v1(f)
-        if tag is not None:
-            f.seek(offset, 2)
-            f.truncate()
+        if delete_v1:
+            tag, offset = _find_id3v1(f)
+            if tag is not None:
+                f.seek(offset, 2)
+                f.truncate()
 
-    # technically an insize=0 tag is invalid, but we delete it anyway
-    # (primarily because we used to write it)
-    if delete_v2:
-        f.seek(0, 0)
-        idata = f.read(10)
-        try:
-            id3, vmaj, vrev, flags, insize = unpack('>3sBBB4s', idata)
-        except struct.error:
-            id3, insize = b'', -1
-        insize = BitPaddedInt(insize)
-        if id3 == b'ID3' and insize >= 0:
-            delete_bytes(f, insize + 10, 0)
+        # technically an insize=0 tag is invalid, but we delete it anyway
+        # (primarily because we used to write it)
+        if delete_v2:
+            f.seek(0, 0)
+            idata = f.read(10)
+            try:
+                id3, vmaj, vrev, flags, insize = unpack('>3sBBB4s', idata)
+            except struct.error:
+                id3, insize = b'', -1
+            insize = BitPaddedInt(insize)
+            if id3 == b'ID3' and insize >= 0:
+                delete_bytes(f, insize + 10, 0)
 
 
 # support open(filename) as interface
@@ -895,7 +955,7 @@ def ParseID3v1(data):
             encoding=0, lang="eng", desc="ID3v1 Comment", text=comment)
     # Don't read a track number if it looks like the comment was
     # padded with spaces instead of nulls (thanks, WinAmp).
-    if track and (track != 32 or data[-3] == b'\x00'[0]):
+    if track and ((track != 32) or (data[-3] == b'\x00'[0])):
         frames["TRCK"] = TRCK(encoding=0, text=str(track))
     if genre != 255:
         frames["TCON"] = TCON(encoding=0, text=str(genre))
@@ -948,15 +1008,16 @@ def MakeID3v1(id3):
         year = b""
     v1["year"] = (year + b"\x00\x00\x00\x00")[:4]
 
-    data = b"TAG"
-    data += v1["title"]
-    data += v1["artist"]
-    data += v1["album"]
-    data += v1["year"]
-    data += v1["comment"]
-    data += v1["track"]
-    data += v1["genre"]
-    return data
+    return (
+        b"TAG" +
+        v1["title"] +
+        v1["artist"] +
+        v1["album"] +
+        v1["year"] +
+        v1["comment"] +
+        v1["track"] +
+        v1["genre"]
+    )
 
 
 class ID3FileType(mutagen.FileType):
@@ -1008,8 +1069,9 @@ class ID3FileType(mutagen.FileType):
         self.filename = filename
         try:
             self.tags = ID3(filename, **kwargs)
-        except error:
+        except ID3NoHeaderError:
             self.tags = None
+
         if self.tags is not None:
             try:
                 offset = self.tags.size
@@ -1017,8 +1079,6 @@ class ID3FileType(mutagen.FileType):
                 offset = None
         else:
             offset = None
-        try:
-            fileobj = open(filename, "rb")
+
+        with open(filename, "rb") as fileobj:
             self.info = self._Info(fileobj, offset)
-        finally:
-            fileobj.close()
