@@ -149,15 +149,10 @@ class MP4Cover(bytes):
 
     def __eq__(self, other):
         if not isinstance(other, MP4Cover):
-            return NotImplemented
+            return bytes(self) == other
 
-        if not bytes.__eq__(self, other):
-            return False
-
-        if self.imageformat != other.imageformat:
-            return False
-
-        return True
+        return (bytes(self) == bytes(other) and
+                self.imageformat == other.imageformat)
 
     def __ne__(self, other):
         return not self.__eq__(other)
@@ -191,18 +186,11 @@ class MP4FreeForm(bytes):
 
     def __eq__(self, other):
         if not isinstance(other, MP4FreeForm):
-            return NotImplemented
+            return bytes(self) == other
 
-        if not bytes.__eq__(self, other):
-            return False
-
-        if self.dataformat != other.dataformat:
-            return False
-
-        if self.version != other.version:
-            return False
-
-        return True
+        return (bytes(self) == bytes(other) and
+                self.dataformat == other.dataformat and
+                self.version == other.version)
 
     def __ne__(self, other):
         return not self.__eq__(other)
@@ -761,19 +749,26 @@ class MP4Tags(DictProxy, Metadata):
         __atoms[name] = (__parse_text, __render_text)
 
     def pprint(self):
+
+        def to_line(key, value):
+            assert isinstance(key, text_type)
+            if isinstance(value, text_type):
+                return u"%s=%s" % (key, value)
+            return u"%s=%r" % (key, value)
+
         values = []
-        for key, value in iteritems(self):
+        for key, value in sorted(iteritems(self)):
             if not isinstance(key, text_type):
                 key = key.decode("latin-1")
             if key == "covr":
-                values.append("%s=%s" % (key, ", ".join(
-                    ["[%d bytes of data]" % len(data) for data in value])))
+                values.append(u"%s=%s" % (key, u", ".join(
+                    [u"[%d bytes of data]" % len(data) for data in value])))
             elif isinstance(value, list):
                 for v in value:
-                    values.append("%s=%r" % (key, v))
+                    values.append(to_line(key, v))
             else:
-                values.append("%s=%r" % (key, value))
-        return "\n".join(values)
+                values.append(to_line(key, value))
+        return u"\n".join(values)
 
 
 class MP4Info(StreamInfo):
