@@ -1,12 +1,10 @@
-# Ogg Vorbis support.
-#
-# Copyright 2006 Joe Wreschnig <piman@sacredchao.net>
+# -*- coding: utf-8 -*-
+
+# Copyright 2006 Joe Wreschnig
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
-#
-# $Id: oggvorbis.py 4275 2008-06-01 06:32:37Z piman $
 
 """Read and write Ogg Vorbis comments.
 
@@ -21,33 +19,45 @@ __all__ = ["OggVorbis", "Open", "delete"]
 
 import struct
 
+from mutagen import StreamInfo
 from mutagen._vorbis import VCommentDict
 from mutagen.ogg import OggPage, OggFileType, error as OggError
 
-class error(OggError): pass
-class OggVorbisHeaderError(error): pass
 
-class OggVorbisInfo(object):
+class error(OggError):
+    pass
+
+
+class OggVorbisHeaderError(error):
+    pass
+
+
+class OggVorbisInfo(StreamInfo):
     """Ogg Vorbis stream information.
 
     Attributes:
-    length - file length in seconds, as a float
-    bitrate - nominal ('average') bitrate in bits per second, as an int
+
+    * length - file length in seconds, as a float
+    * bitrate - nominal ('average') bitrate in bits per second, as an int
     """
 
     length = 0
 
     def __init__(self, fileobj):
         page = OggPage(fileobj)
-        while not page.packets[0].startswith("\x01vorbis"):
+        while not page.packets[0].startswith(b"\x01vorbis"):
             page = OggPage(fileobj)
         if not page.first:
             raise OggVorbisHeaderError(
                 "page has ID header, but doesn't start a stream")
         (self.channels, self.sample_rate, max_bitrate, nominal_bitrate,
-         min_bitrate) = struct.unpack("<B4I", page.packets[0][11:28])
+         min_bitrate) = struct.unpack("<B4i", page.packets[0][11:28])
         self.serial = page.serial
 
+        max_bitrate = max(0, max_bitrate)
+        min_bitrate = max(0, min_bitrate)
+        nominal_bitrate = max(0, nominal_bitrate)
+
         if nominal_bitrate == 0:
             self.bitrate = (max_bitrate + min_bitrate) // 2
         elif max_bitrate and max_bitrate < nominal_bitrate:
@@ -59,8 +69,14 @@ class OggVorbisInfo(object):
         else:
             self.bitrate = nominal_bitrate
 
+    def _post_tags(self, fileobj):
+        page = OggPage.find_last(fileobj, self.serial)
+        self.length = page.position / float(self.sample_rate)
+
     def pprint(self):
-        return "Ogg Vorbis, %.2f seconds, %d bps" % (self.length, self.bitrate)
+        return u"Ogg Vorbis, %.2f seconds, %d bps" % (
+            self.length, self.bitrate)
+
 
 class OggVCommentDict(VCommentDict):
     """Vorbis comments embedded in an Ogg bitstream."""
@@ -73,7 +89,7 @@ class OggVCommentDict(VCommentDict):
             if page.serial == info.serial:
                 pages.append(page)
                 complete = page.complete or (len(page.packets) > 1)
-        data = OggPage.to_packets(pages)[0][7:] # Strip off "\x03vorbis".
+        data = OggPage.to_packets(pages)[0][7:]  # Strip off "\x03vorbis".
         super(OggVCommentDict, self).__init__(data)
 
     def _inject(self, fileobj):
@@ -83,7 +99,7 @@ class OggVCommentDict(VCommentDict):
         # plus grab any stray setup packet data out of them.
         fileobj.seek(0)
         page = OggPage(fileobj)
-        while not page.packets[0].startswith("\x03vorbis"):
+        while not page.packets[0].startswith(b"\x03vorbis"):
             page = OggPage(fileobj)
 
         old_pages = [page]
@@ -95,11 +111,12 @@ class OggVCommentDict(VCommentDict):
         packets = OggPage.to_packets(old_pages, strict=False)
 
         # Set the new comment packet.
-        packets[0] = "\x03vorbis" + self.write()
+        packets[0] = b"\x03vorbis" + self.write()
 
         new_pages = OggPage.from_packets(packets, old_pages[0].sequence)
         OggPage.replace(fileobj, old_pages, new_pages)
 
+
 class OggVorbis(OggFileType):
     """An Ogg Vorbis file."""
 
@@ -108,12 +125,15 @@ class OggVorbis(OggFileType):
     _Error = OggVorbisHeaderError
     _mimes = ["audio/vorbis", "audio/x-vorbis"]
 
+    @staticmethod
     def score(filename, fileobj, header):
-        return (header.startswith("OggS") * ("\x01vorbis" in header))
-    score = staticmethod(score)
+        return (header.startswith(b"OggS") * (b"\x01vorbis" in header))
+
 
 Open = OggVorbis
 
+
 def delete(filename):
     """Remove tags from a file."""
+
     OggVorbis(filename).delete()
