@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # FIXME: This test suite is a mess, a lot of it dates from PyMusepack so
 # it doesn't match the other Mutagen test conventions/quality.
 
@@ -6,19 +7,19 @@ import shutil
 
 from tempfile import mkstemp
 
-from tests import TestCase, add
+from tests import TestCase, DATA_DIR
 
 import mutagen.apev2
 from mutagen._compat import PY3, text_type
-from mutagen.apev2 import APEv2File, APEv2, is_valid_apev2_key
+from mutagen.apev2 import APEv2File, APEv2, is_valid_apev2_key, APEBadItemError
 
 
-DIR = os.path.dirname(__file__)
-SAMPLE = os.path.join(DIR, "data", "click.mpc")
-OLD = os.path.join(DIR, "data", "oldtag.apev2")
-BROKEN = os.path.join(DIR, "data", "brokentag.apev2")
-LYRICS2 = os.path.join(DIR, "data", "apev2-lyricsv2.mp3")
-INVAL_ITEM_COUNT = os.path.join(DIR, "data", "145-invalid-item-count.apev2")
+SAMPLE = os.path.join(DATA_DIR, "click.mpc")
+OLD = os.path.join(DATA_DIR, "oldtag.apev2")
+BROKEN = os.path.join(DATA_DIR, "brokentag.apev2")
+LYRICS2 = os.path.join(DATA_DIR, "apev2-lyricsv2.mp3")
+INVAL_ITEM_COUNT = os.path.join(DATA_DIR, "145-invalid-item-count.apev2")
+
 
 class Tis_valid_apev2_key(TestCase):
 
@@ -34,8 +35,6 @@ class Tis_valid_apev2_key(TestCase):
         def test_py3(self):
             self.assertRaises(TypeError, is_valid_apev2_key, b"abc")
 
-add(Tis_valid_apev2_key)
-
 
 class TAPEInvalidItemCount(TestCase):
     # http://code.google.com/p/mutagen/issues/detail?id=145
@@ -44,8 +43,6 @@ class TAPEInvalidItemCount(TestCase):
         x = mutagen.apev2.APEv2(INVAL_ITEM_COUNT)
         self.failUnlessEqual(len(x.keys()), 17)
 
-add(TAPEInvalidItemCount)
-
 
 class TAPEWriter(TestCase):
     offset = 0
@@ -68,7 +65,7 @@ class TAPEWriter(TestCase):
         self.tag = mutagen.apev2.APEv2(SAMPLE + ".new")
 
     def test_changed(self):
-        size = os.path.getsize(SAMPLE + ".new") 
+        size = os.path.getsize(SAMPLE + ".new")
         self.tag.save()
         self.failUnlessEqual(
             os.path.getsize(SAMPLE + ".new"), size - self.offset)
@@ -80,7 +77,7 @@ class TAPEWriter(TestCase):
         tag = mutagen.apev2.APEv2(BROKEN)
         tag.save(BROKEN + ".new")
         self.failUnlessEqual(
-            os.path.getsize(OLD), os.path.getsize(BROKEN+".new"))
+            os.path.getsize(OLD), os.path.getsize(BROKEN + ".new"))
 
     def test_readback(self):
         for k, v in self.tag.items():
@@ -103,7 +100,7 @@ class TAPEWriter(TestCase):
     def test_empty(self):
         self.failUnlessRaises(
             IOError, mutagen.apev2.APEv2,
-            os.path.join("tests", "data", "emptyfile.mp3"))
+            os.path.join(DATA_DIR, "emptyfile.mp3"))
 
     def test_tag_at_start(self):
         filename = SAMPLE + ".tag_at_start"
@@ -150,7 +147,6 @@ class TAPEWriter(TestCase):
         os.unlink(SAMPLE + ".justtag")
         os.unlink(SAMPLE + ".tag_at_start")
 
-add(TAPEWriter)
 
 class TAPEv2ThenID3v1Writer(TAPEWriter):
     offset = 128
@@ -170,7 +166,6 @@ class TAPEv2ThenID3v1Writer(TAPEWriter):
     def test_tag_at_start_write(self):
         pass
 
-add(TAPEv2ThenID3v1Writer)
 
 class TAPEv2(TestCase):
 
@@ -214,14 +209,14 @@ class TAPEv2(TestCase):
 
     def test_module_delete_empty(self):
         from mutagen.apev2 import delete
-        delete(os.path.join("tests", "data", "emptyfile.mp3"))
+        delete(os.path.join(DATA_DIR, "emptyfile.mp3"))
 
     def test_invalid(self):
         self.failUnlessRaises(IOError, mutagen.apev2.APEv2, "dne")
 
     def test_no_tag(self):
         self.failUnlessRaises(IOError, mutagen.apev2.APEv2,
-                              os.path.join("tests", "data", "empty.mp3"))
+                              os.path.join(DATA_DIR, "empty.mp3"))
 
     def test_cases(self):
         self.failUnlessEqual(self.audio["artist"], self.audio["ARTIST"])
@@ -276,7 +271,6 @@ class TAPEv2(TestCase):
     def tearDown(self):
         os.unlink(self.filename)
 
-add(TAPEv2)
 
 class TAPEv2ThenID3v1(TAPEv2):
 
@@ -287,7 +281,6 @@ class TAPEv2ThenID3v1(TAPEv2):
         f.close()
         self.audio = APEv2(self.filename)
 
-add(TAPEv2ThenID3v1)
 
 class TAPEv2WithLyrics2(TestCase):
 
@@ -299,7 +292,6 @@ class TAPEv2WithLyrics2(TestCase):
         self.failUnlessEqual(self.tag["REPLAYGAIN_TRACK_GAIN"], "-4.080000 dB")
         self.failUnlessEqual(self.tag["REPLAYGAIN_TRACK_PEAK"], "1.008101")
 
-add(TAPEv2WithLyrics2)
 
 class TAPEBinaryValue(TestCase):
 
@@ -308,7 +300,7 @@ class TAPEBinaryValue(TestCase):
 
     def setUp(self):
         self.sample = b"\x12\x45\xde"
-        self.value = mutagen.apev2.APEValue(self.sample,mutagen.apev2.BINARY)
+        self.value = mutagen.apev2.APEValue(self.sample, mutagen.apev2.BINARY)
 
     def test_type(self):
         self.failUnless(isinstance(self.value, self.BV))
@@ -320,13 +312,12 @@ class TAPEBinaryValue(TestCase):
         repr(self.value)
 
     def test_pprint(self):
-        self.value.pprint()
+        self.assertEqual(self.value.pprint(), "[3 bytes]")
 
-    def test_type(self):
+    def test_type2(self):
         self.assertRaises(TypeError,
                           mutagen.apev2.APEValue, u"abc", mutagen.apev2.BINARY)
 
-add(TAPEBinaryValue)
 
 class TAPETextValue(TestCase):
 
@@ -338,9 +329,18 @@ class TAPETextValue(TestCase):
         self.value = mutagen.apev2.APEValue(
             "\0".join(self.sample), mutagen.apev2.TEXT)
 
+    def test_parse(self):
+        self.assertRaises(APEBadItemError, self.TV._new, b"\xff")
+
     def test_type(self):
         self.failUnless(isinstance(self.value, self.TV))
 
+    def test_construct(self):
+        self.assertEqual(text_type(self.TV(u"foo")), u"foo")
+        if not PY3:
+            self.assertEqual(text_type(self.TV(b"foo")), u"foo")
+            self.assertRaises(ValueError, self.TV, b"\xff")
+
     def test_list(self):
         self.failUnlessEqual(self.sample, list(self.value))
 
@@ -354,8 +354,21 @@ class TAPETextValue(TestCase):
         for i in range(len(self.value)):
             self.failUnlessEqual(self.sample[i], self.value[i])
 
-    if PY3:
-        def test_py3(self):
+    def test_delitem(self):
+        del self.sample[1]
+        self.assertEqual(list(self.sample), ["foo", "baz"])
+        del self.sample[1:]
+        self.assertEqual(list(self.sample), ["foo"])
+
+    def test_insert(self):
+        self.sample.insert(0, "a")
+        self.assertEqual(len(self.sample), 4)
+        self.assertEqual(self.sample[0], "a")
+        if PY3:
+            self.assertRaises(TypeError, self.value.insert, 2, b"abc")
+
+    def test_types(self):
+        if PY3:
             self.assertRaises(TypeError, self.value.__setitem__, 2, b"abc")
             self.assertRaises(
                 TypeError, mutagen.apev2.APEValue, b"abc", mutagen.apev2.TEXT)
@@ -363,7 +376,12 @@ class TAPETextValue(TestCase):
     def test_repr(self):
         repr(self.value)
 
-add(TAPETextValue)
+    def test_str(self):
+        self.assertEqual(text_type(self.value), u"foo\x00bar\x00baz")
+
+    def test_pprint(self):
+        self.assertEqual(self.value.pprint(), "foo / bar / baz")
+
 
 class TAPEExtValue(TestCase):
 
@@ -391,14 +409,17 @@ class TAPEExtValue(TestCase):
                 mutagen.apev2.EXTERNAL)
 
     def test_pprint(self):
-        self.value.pprint()
+        self.assertEqual(self.value.pprint(), "[External] http://foo")
 
-add(TAPEExtValue)
 
 class TAPEv2File(TestCase):
 
     def setUp(self):
-        self.audio = APEv2File("tests/data/click.mpc")
+        self.audio = APEv2File(os.path.join(DATA_DIR, "click.mpc"))
+
+    def test_empty(self):
+        f = APEv2File(os.path.join(DATA_DIR, "xing.mp3"))
+        self.assertFalse(f.items())
 
     def test_add_tags(self):
         self.failUnless(self.audio.tags is None)
@@ -409,5 +430,3 @@ class TAPEv2File(TestCase):
     def test_unknown_info(self):
         info = self.audio.info
         info.pprint()
-
-add(TAPEv2File)
