@@ -1,9 +1,11 @@
+# -*- coding: utf-8 -*-
+
 import os
 import shutil
 from tempfile import mkstemp
-from tests import TestCase, add
+from tests import TestCase, DATA_DIR
 
-from mutagen._compat import PY3, text_type
+from mutagen._compat import PY3, text_type, PY2
 from mutagen.asf import ASF, ASFHeaderError, ASFValue, UNICODE, DWORD, QWORD
 from mutagen.asf import BOOL, WORD, BYTEARRAY, GUID
 from mutagen.asf import ASFUnicodeAttribute, ASFError, ASFByteArrayAttribute, \
@@ -16,23 +18,21 @@ class TASFFile(TestCase):
     def test_not_my_file(self):
         self.failUnlessRaises(
             ASFHeaderError, ASF,
-            os.path.join("tests", "data", "empty.ogg"))
+            os.path.join(DATA_DIR, "empty.ogg"))
         self.failUnlessRaises(
             ASFHeaderError, ASF,
-            os.path.join("tests", "data", "click.mpc"))
-
-add(TASFFile)
+            os.path.join(DATA_DIR, "click.mpc"))
 
 
 class TASFInfo(TestCase):
 
     def setUp(self):
         # WMA 9.1 64kbps CBR 48khz
-        self.wma1 = ASF(os.path.join("tests", "data", "silence-1.wma"))
+        self.wma1 = ASF(os.path.join(DATA_DIR, "silence-1.wma"))
         # WMA 9.1 Professional 192kbps VBR 44khz
-        self.wma2 = ASF(os.path.join("tests", "data", "silence-2.wma"))
+        self.wma2 = ASF(os.path.join(DATA_DIR, "silence-2.wma"))
         # WMA 9.1 Lossless 44khz
-        self.wma3 = ASF(os.path.join("tests", "data", "silence-3.wma"))
+        self.wma3 = ASF(os.path.join(DATA_DIR, "silence-3.wma"))
 
     def test_length(self):
         self.failUnlessAlmostEqual(self.wma1.info.length, 3.7, 1)
@@ -54,8 +54,6 @@ class TASFInfo(TestCase):
         self.failUnlessEqual(self.wma2.info.channels, 2)
         self.failUnlessEqual(self.wma3.info.channels, 2)
 
-add(TASFInfo)
-
 
 class TASF(TestCase):
 
@@ -68,6 +66,9 @@ class TASF(TestCase):
     def tearDown(self):
         os.unlink(self.filename)
 
+
+class TASFMixin(object):
+
     def test_pprint(self):
         self.failUnless(self.audio.pprint())
 
@@ -86,6 +87,20 @@ class TASF(TestCase):
         else:
             self.failUnlessEqual(self.audio[key], result or value)
 
+    def test_slice(self):
+        tags = self.audio.tags
+        tags.clear()
+        tags["Author"] = [u"Foo", u"Bar"]
+        self.assertEqual(tags[:], [("Author", "Foo"), ("Author", "Bar")])
+        del tags[:]
+        self.assertEqual(tags[:], [])
+        tags[:] = [("Author", "Baz")]
+        self.assertEqual(tags.items(), [("Author", ["Baz"])])
+
+    def test_iter(self):
+        self.assertEqual(next(iter(self.audio.tags)), ("Title", "test"))
+        self.assertEqual(list(self.audio.tags)[0], ("Title", "test"))
+
     def test_contains(self):
         self.failUnlessEqual("notatag" in self.audio.tags, False)
 
@@ -99,6 +114,22 @@ class TASF(TestCase):
         value = ASFValue(b'\x9eZl}\x89\xa2\xb5D\xb8\xa30\xfe', GUID)
         self.set_key(u"WM/WMCollectionGroupID", value, [value])
 
+    def test_py3_bytes(self):
+        if PY3:
+            value = ASFValue(b'\xff\x00', BYTEARRAY)
+            self.set_key(u"QL/Something", [b'\xff\x00'], [value])
+
+    def test_set_invalid(self):
+        setitem = self.audio.__setitem__
+        if PY2:
+            self.assertRaises(ValueError, setitem, u"QL/Something", [b"\xff"])
+        self.assertRaises(TypeError, setitem, u"QL/Something", [object()])
+
+        # don't delete on error
+        setitem(u"QL/Foobar", [u"ok"])
+        self.assertRaises(TypeError, setitem, u"QL/Foobar", [object()])
+        self.assertEqual(self.audio[u"QL/Foobar"], [u"ok"])
+
     def test_auto_unicode(self):
         self.set_key(u"WM/AlbumTitle", u"foo",
                      [ASFValue(u"foo", UNICODE)])
@@ -162,8 +193,8 @@ class TASF(TestCase):
         self.audio.save()
         self.audio = ASF(self.audio.filename)
         self.failUnlessEqual(self.audio["QL/NoStream"][0].stream, None)
-        self.failUnlessEqual(self.audio["QL/OneHasStream"][0].stream, 2)
-        self.failUnlessEqual(self.audio["QL/OneHasStream"][1].stream, None)
+        self.failUnlessEqual(self.audio["QL/OneHasStream"][1].stream, 2)
+        self.failUnlessEqual(self.audio["QL/OneHasStream"][0].stream, None)
         self.failUnlessEqual(self.audio["QL/AllHaveStream"][0].stream, 1)
         self.failUnlessEqual(self.audio["QL/AllHaveStream"][1].stream, 2)
 
@@ -182,8 +213,8 @@ class TASF(TestCase):
         self.audio.save()
         self.audio = ASF(self.audio.filename)
         self.failUnlessEqual(self.audio["QL/NoLang"][0].language, None)
-        self.failUnlessEqual(self.audio["QL/OneHasLang"][0].language, 2)
-        self.failUnlessEqual(self.audio["QL/OneHasLang"][1].language, None)
+        self.failUnlessEqual(self.audio["QL/OneHasLang"][1].language, 2)
+        self.failUnlessEqual(self.audio["QL/OneHasLang"][0].language, None)
         self.failUnlessEqual(self.audio["QL/AllHaveLang"][0].language, 1)
         self.failUnlessEqual(self.audio["QL/AllHaveLang"][1].language, 2)
 
@@ -196,14 +227,15 @@ class TASF(TestCase):
             ]
         self.audio.save()
         self.audio = ASF(self.audio.filename)
+        # order not preserved here because they end up in different objects.
+        self.failUnlessEqual(self.audio["QL/Mix"][1].language, None)
+        self.failUnlessEqual(self.audio["QL/Mix"][1].stream, 1)
+        self.failUnlessEqual(self.audio["QL/Mix"][2].language, 2)
+        self.failUnlessEqual(self.audio["QL/Mix"][2].stream, 0)
+        self.failUnlessEqual(self.audio["QL/Mix"][3].language, 4)
+        self.failUnlessEqual(self.audio["QL/Mix"][3].stream, 3)
         self.failUnlessEqual(self.audio["QL/Mix"][0].language, None)
-        self.failUnlessEqual(self.audio["QL/Mix"][0].stream, 1)
-        self.failUnlessEqual(self.audio["QL/Mix"][1].language, 2)
-        self.failUnlessEqual(self.audio["QL/Mix"][1].stream, 0)
-        self.failUnlessEqual(self.audio["QL/Mix"][2].language, 4)
-        self.failUnlessEqual(self.audio["QL/Mix"][2].stream, 3)
-        self.failUnlessEqual(self.audio["QL/Mix"][3].language, None)
-        self.failUnlessEqual(self.audio["QL/Mix"][3].stream, None)
+        self.failUnlessEqual(self.audio["QL/Mix"][0].stream, None)
 
     def test_data_size(self):
         v = ASFValue("", UNICODE, data=b'4\xd8\x1e\xdd\x00\x00')
@@ -222,16 +254,33 @@ class TASFAttributes(TestCase):
 
         self.assertRaises(ASFError, ASFUnicodeAttribute, data=b"\x00")
         self.assertEqual(ASFUnicodeAttribute(u"foo").value, u"foo")
-        self.assertEqual(
-            bytes(ASFUnicodeAttribute(u"foo")), b"f\x00o\x00o\x00")
-        self.assertEqual(
-            text_type(ASFUnicodeAttribute(u"foo")), u"foo")
+
+    def test_ASFUnicodeAttribute_dunder(self):
+        attr = ASFUnicodeAttribute(u"foo")
+
+        self.assertEqual(bytes(attr), b"f\x00o\x00o\x00")
+        self.assertEqual(text_type(attr), u"foo")
+        if PY3:
+            self.assertEqual(repr(attr), "ASFUnicodeAttribute('foo')")
+        else:
+            self.assertEqual(repr(attr), "ASFUnicodeAttribute(u'foo')")
+        self.assertRaises(TypeError, int, attr)
 
     def test_ASFByteArrayAttribute(self):
-        self.assertEqual(ASFByteArrayAttribute(data=b"\xff").value, b"\xff")
         self.assertRaises(TypeError, ASFByteArrayAttribute, u"foo")
+        self.assertEqual(ASFByteArrayAttribute(data=b"\xff").value, b"\xff")
 
-    def test_compat(self):
+    def test_ASFByteArrayAttribute_dunder(self):
+        attr = ASFByteArrayAttribute(data=b"\xff")
+        self.assertEqual(bytes(attr), b"\xff")
+        self.assertEqual(text_type(attr), u"[binary data (1 bytes)]")
+        if PY3:
+            self.assertEqual(repr(attr), r"ASFByteArrayAttribute(b'\xff')")
+        else:
+            self.assertEqual(repr(attr), r"ASFByteArrayAttribute('\xff')")
+        self.assertRaises(TypeError, int, attr)
+
+    def test_ASFByteArrayAttribute_compat(self):
         ba = ASFByteArrayAttribute()
         ba.value = b"\xff"
         self.assertEqual(ba._render(), b"\xff")
@@ -240,6 +289,17 @@ class TASFAttributes(TestCase):
         self.assertEqual(ASFGUIDAttribute(data=b"\xff").value, b"\xff")
         self.assertRaises(TypeError, ASFGUIDAttribute, u"foo")
 
+    def test_ASFGUIDAttribute_dunder(self):
+        attr = ASFGUIDAttribute(data=b"\xff")
+        self.assertEqual(bytes(attr), b"\xff")
+        if PY3:
+            self.assertEqual(text_type(attr), u"b'\\xff'")
+            self.assertEqual(repr(attr), "ASFGUIDAttribute(b'\\xff')")
+        else:
+            self.assertEqual(text_type(attr), u"'\\xff'")
+            self.assertEqual(repr(attr), "ASFGUIDAttribute('\\xff')")
+        self.assertRaises(TypeError, int, attr)
+
     def test_ASFBoolAttribute(self):
         self.assertEqual(
             ASFBoolAttribute(data=b"\x01\x00\x00\x00").value, True)
@@ -247,6 +307,13 @@ class TASFAttributes(TestCase):
             ASFBoolAttribute(data=b"\x00\x00\x00\x00").value, False)
         self.assertEqual(ASFBoolAttribute(False).value, False)
 
+    def test_ASFBoolAttribute_dunder(self):
+        attr = ASFBoolAttribute(False)
+        self.assertEqual(bytes(attr), b"False")
+        self.assertEqual(text_type(attr), u"False")
+        self.assertEqual(repr(attr), "ASFBoolAttribute(False)")
+        self.assertRaises(TypeError, int, attr)
+
     def test_ASFWordAttribute(self):
         self.assertEqual(
             ASFWordAttribute(data=b"\x00" * 2).value, 0)
@@ -255,6 +322,13 @@ class TASFAttributes(TestCase):
         self.assertRaises(ValueError, ASFWordAttribute, -1)
         self.assertRaises(ValueError, ASFWordAttribute, 2 ** 16)
 
+    def test_ASFWordAttribute_dunder(self):
+        attr = ASFWordAttribute(data=b"\x00" * 2)
+        self.assertEqual(bytes(attr), b"0")
+        self.assertEqual(text_type(attr), u"0")
+        self.assertEqual(repr(attr), "ASFWordAttribute(0)")
+        self.assertEqual(int(attr), 0)
+
     def test_ASFDWordAttribute(self):
         self.assertEqual(
             ASFDWordAttribute(data=b"\x00" * 4).value, 0)
@@ -263,6 +337,13 @@ class TASFAttributes(TestCase):
         self.assertRaises(ValueError, ASFDWordAttribute, -1)
         self.assertRaises(ValueError, ASFDWordAttribute, 2 ** 32)
 
+    def test_ASFDWordAttribute_dunder(self):
+        attr = ASFDWordAttribute(data=b"\x00" * 4)
+        self.assertEqual(bytes(attr), b"0")
+        self.assertEqual(text_type(attr), u"0")
+        self.assertEqual(repr(attr).replace("0L", "0"), "ASFDWordAttribute(0)")
+        self.assertEqual(int(attr), 0)
+
     def test_ASFQWordAttribute(self):
         self.assertEqual(
             ASFQWordAttribute(data=b"\x00" * 8).value, 0)
@@ -271,27 +352,28 @@ class TASFAttributes(TestCase):
         self.assertRaises(ValueError, ASFQWordAttribute, -1)
         self.assertRaises(ValueError, ASFQWordAttribute, 2 ** 64)
 
-
-add(TASFAttributes)
+    def test_ASFQWordAttribute_dunder(self):
+        attr = ASFQWordAttribute(data=b"\x00" * 8)
+        self.assertEqual(bytes(attr), b"0")
+        self.assertEqual(text_type(attr), u"0")
+        self.assertEqual(repr(attr).replace("0L", "0"), "ASFQWordAttribute(0)")
+        self.assertEqual(int(attr), 0)
 
 
-class TASFTags1(TASF):
-    original = os.path.join("tests", "data", "silence-1.wma")
-add(TASFTags1)
+class TASFTags1(TASF, TASFMixin):
+    original = os.path.join(DATA_DIR, "silence-1.wma")
 
 
-class TASFTags2(TASF):
-    original = os.path.join("tests", "data", "silence-2.wma")
-add(TASFTags2)
+class TASFTags2(TASF, TASFMixin):
+    original = os.path.join(DATA_DIR, "silence-2.wma")
 
 
-class TASFTags3(TASF):
-    original = os.path.join("tests", "data", "silence-3.wma")
-add(TASFTags3)
+class TASFTags3(TASF, TASFMixin):
+    original = os.path.join(DATA_DIR, "silence-3.wma")
 
 
 class TASFIssue29(TestCase):
-    original = os.path.join("tests", "data", "issue_29.wma")
+    original = os.path.join(DATA_DIR, "issue_29.wma")
 
     def setUp(self):
         fd, self.filename = mkstemp(suffix='wma')
@@ -302,6 +384,9 @@ class TASFIssue29(TestCase):
     def tearDown(self):
         os.unlink(self.filename)
 
+    def test_pprint(self):
+        self.audio.pprint()
+
     def test_issue_29_description(self):
         self.audio["Description"] = "Hello"
         self.audio.save()
@@ -313,12 +398,89 @@ class TASFIssue29(TestCase):
         audio.save()
         audio = ASF(self.filename)
         self.failIf("Description" in audio)
-add(TASFIssue29)
+
+
+class TASFAttrDest(TestCase):
+
+    original = os.path.join(DATA_DIR, "silence-1.wma")
+
+    def setUp(self):
+        fd, self.filename = mkstemp(suffix='wma')
+        os.close(fd)
+        shutil.copy(self.original, self.filename)
+        audio = ASF(self.filename)
+        audio.clear()
+        audio.save()
+
+    def tearDown(self):
+        os.unlink(self.filename)
+
+    def test_author(self):
+        audio = ASF(self.filename)
+        values = [u"Foo", u"Bar", u"Baz"]
+        audio["Author"] = values
+        audio.save()
+        self.assertEqual(
+            list(audio.to_content_description.items()), [(u"Author", u"Foo")])
+        self.assertEqual(
+            audio.to_metadata_library,
+            [(u"Author", u"Bar"), (u"Author", u"Baz")])
+
+        new = ASF(self.filename)
+        self.assertEqual(new["Author"], values)
+
+    def test_author_long(self):
+        audio = ASF(self.filename)
+        # 2 ** 16 - 2 bytes encoded text + 2 bytes termination
+        just_small_enough = u"a" * (((2 ** 16) // 2) - 2)
+        audio["Author"] = [just_small_enough]
+        audio.save()
+        self.assertTrue(audio.to_content_description)
+        self.assertFalse(audio.to_metadata_library)
+
+        audio["Author"] = [just_small_enough + u"a"]
+        audio.save()
+        self.assertFalse(audio.to_content_description)
+        self.assertTrue(audio.to_metadata_library)
+
+    def test_multi_order(self):
+        audio = ASF(self.filename)
+        audio["Author"] = [u"a", u"b", u"c"]
+        audio.save()
+        audio = ASF(self.filename)
+        self.assertEqual(audio["Author"], [u"a", u"b", u"c"])
+
+    def test_multi_order_extended(self):
+        audio = ASF(self.filename)
+        audio["WM/Composer"] = [u"a", u"b", u"c"]
+        audio.save()
+        audio = ASF(self.filename)
+        self.assertEqual(audio["WM/Composer"], [u"a", u"b", u"c"])
+
+    def test_non_text_type(self):
+        audio = ASF(self.filename)
+        audio["Author"] = [42]
+        audio.save()
+        self.assertFalse(audio.to_content_description)
+        new = ASF(self.filename)
+        self.assertEqual(new["Author"], [42])
+
+    def test_empty(self):
+        audio = ASF(self.filename)
+        audio["Author"] = [u"", u""]
+        audio["Title"] = [u""]
+        audio["Copyright"] = []
+        audio.save()
+
+        new = ASF(self.filename)
+        self.assertEqual(new["Author"], [u"", u""])
+        self.assertEqual(new["Title"], [u""])
+        self.assertFalse("Copyright" in new)
 
 
 class TASFLargeValue(TestCase):
 
-    original = os.path.join("tests", "data", "silence-1.wma")
+    original = os.path.join(DATA_DIR, "silence-1.wma")
 
     def setUp(self):
         fd, self.filename = mkstemp(suffix='wma')
@@ -371,13 +533,11 @@ class TASFLargeValue(TestCase):
         self.failIf("QL/GuidObject" in audio.to_metadata)
         self.failIf("QL/GuidObject" not in dict(audio.to_metadata_library))
 
-add(TASFLargeValue)
-
 
 class TASFUpdateSize(TestCase):
     # http://code.google.com/p/mutagen/issues/detail?id=81#c4
 
-    original = os.path.join("tests", "data", "silence-1.wma")
+    original = os.path.join(DATA_DIR, "silence-1.wma")
 
     def setUp(self):
         fd, self.filename = mkstemp(suffix='wma')
@@ -395,5 +555,3 @@ class TASFUpdateSize(TestCase):
         for tag in audio.keys():
             del(audio[tag])
             audio.save()
-
-add(TASFUpdateSize)
