@@ -3,6 +3,7 @@
 import shutil
 import os
 import subprocess
+from tempfile import mkstemp
 
 from tests import TestCase, DATA_DIR
 
@@ -267,14 +268,16 @@ class TPadding(TestCase):
 
 class TFLAC(TestCase):
     SAMPLE = os.path.join(DATA_DIR, "silence-44-s.flac")
-    NEW = SAMPLE + ".new"
 
     def setUp(self):
+        fd, self.NEW = mkstemp(".flac")
+        os.close(fd)
         shutil.copy(self.SAMPLE, self.NEW)
-        self.failUnlessEqual(open(self.SAMPLE, "rb").read(),
-                             open(self.NEW, "rb").read())
         self.flac = FLAC(self.NEW)
 
+    def tearDown(self):
+        os.unlink(self.NEW)
+
     def test_delete(self):
         self.failUnless(self.flac.tags)
         self.flac.delete()
@@ -489,9 +492,6 @@ class TFLAC(TestCase):
     def test_load_flac_with_application_block(self):
         FLAC(os.path.join(DATA_DIR, "flac_application.flac"))
 
-    def tearDown(self):
-        os.unlink(self.NEW)
-
 
 class TFLACFile(TestCase):
 
@@ -526,11 +526,15 @@ class TFLACBadBlockSize(TestCase):
 
 class TFLACBadBlockSizeWrite(TestCase):
     TOO_SHORT = os.path.join(DATA_DIR, "52-too-short-block-size.flac")
-    NEW = TOO_SHORT + ".new"
 
     def setUp(self):
+        fd, self.NEW = mkstemp(".flac")
+        os.close(fd)
         shutil.copy(self.TOO_SHORT, self.NEW)
 
+    def tearDown(self):
+        os.unlink(self.NEW)
+
     def test_write_reread(self):
         flac = FLAC(self.NEW)
         del(flac["artist"])
@@ -540,8 +544,48 @@ class TFLACBadBlockSizeWrite(TestCase):
         data = open(self.NEW, "rb").read(1024)
         self.failIf(b"Tunng" in data)
 
+
+class TFLACBadBlockSizeOverflow(TestCase):
+
+    def setUp(self):
+        fd, self.filename = mkstemp(".flac")
+        os.close(fd)
+        shutil.copy(os.path.join(DATA_DIR, "silence-44-s.flac"), self.filename)
+
     def tearDown(self):
-        os.unlink(self.NEW)
+        os.unlink(self.filename)
+
+    def test_largest_valid(self):
+        f = FLAC(self.filename)
+        pic = Picture()
+        pic.data = b"\x00" * (2 ** 24 - 32)
+        self.assertEqual(len(pic.write()), 2 ** 24)
+        f.add_picture(pic)
+        f.save()
+
+    def test_smallest_invalid(self):
+        f = FLAC(self.filename)
+        pic = Picture()
+        pic.data = b"\x00" * (2 ** 24 - 31)
+        f.add_picture(pic)
+        self.assertRaises(error, f.save)
+
+    def test_invalid_overflow_recover_and_save_back(self):
+        # save a picture which is too large for flac, but still write it
+        # with a wrong block size
+        f = FLAC(self.filename)
+        f.clear_pictures()
+        pic = Picture()
+        pic.data = b"\x00" * (2 ** 24 - 31)
+        pic._invalid_overflow_size = 42
+        f.add_picture(pic)
+        f.save()
+
+        # make sure we can read it and save it again
+        f = FLAC(self.filename)
+        self.assertTrue(f.pictures)
+        self.assertEqual(len(f.pictures[0].data), 2 ** 24 - 31)
+        f.save()
 
 
 class CVE20074619(TestCase):
