@@ -1,19 +1,21 @@
+# -*- coding: utf-8 -*-
+
 import os
 import random
 import shutil
+import subprocess
 
 from mutagen._compat import BytesIO
-from tests import TestCase, add
+from tests import TestCase, DATA_DIR
 from mutagen.ogg import OggPage, error as OggError
 from mutagen._util import cdata
 from tempfile import mkstemp
-try: from os.path import devnull
-except ImportError: devnull = "/dev/null"
+
 
 class TOggPage(TestCase):
 
     def setUp(self):
-        self.fileobj = open(os.path.join("tests", "data", "empty.ogg"), "rb")
+        self.fileobj = open(os.path.join(DATA_DIR, "empty.ogg"), "rb")
         self.page = OggPage(self.fileobj)
 
         pages = [OggPage(), OggPage(), OggPage()]
@@ -74,7 +76,8 @@ class TOggPage(TestCase):
 
     def test_wiggle_room(self):
         packets = [b"1" * 511, b"2" * 511, b"3" * 511]
-        pages = OggPage.from_packets(packets, default_size=510, wiggle_room=100)
+        pages = OggPage.from_packets(
+            packets, default_size=510, wiggle_room=100)
         self.failUnlessEqual(len(pages), 3)
         self.failUnlessEqual(OggPage.to_packets(pages), packets)
 
@@ -122,7 +125,7 @@ class TOggPage(TestCase):
         try:
             fd, filename = mkstemp(suffix=".ogg")
             os.close(fd)
-            shutil.copy(os.path.join("tests", "data", "multipagecomment.ogg"),
+            shutil.copy(os.path.join(DATA_DIR, "multipagecomment.ogg"),
                         filename)
             fileobj = open(filename, "rb+")
             OggPage.renumber(fileobj, 1002429366, 20)
@@ -131,8 +134,10 @@ class TOggPage(TestCase):
             OggPage.renumber(fileobj, 1002429366, 0)
             fileobj.close()
         finally:
-            try: os.unlink(filename)
-            except OSError: pass
+            try:
+                os.unlink(filename)
+            except OSError:
+                pass
 
     def test_renumber_muxed(self):
         pages = [OggPage() for i in range(10)]
@@ -204,7 +209,8 @@ class TOggPage(TestCase):
         self.failUnlessEqual(OggPage.to_packets(pages), packets)
 
     def test_random_data_roundtrip(self):
-        try: random_file = open("/dev/urandom", "rb")
+        try:
+            random_file = open("/dev/urandom", "rb")
         except (IOError, OSError):
             print("WARNING: Random data round trip test disabled.")
             return
@@ -255,7 +261,7 @@ class TOggPage(TestCase):
         packets = [b"1"] * 3000
         pages = OggPage.from_packets(packets)
         map(OggPage.write, pages)
-        self.failUnless(len(pages) > 3000/255)
+        self.failUnless(len(pages) > 3000 // 255)
 
     def test_read_max_size(self):
         page = OggPage()
@@ -290,7 +296,8 @@ class TOggPage(TestCase):
 
     def test_find_last(self):
         pages = [OggPage() for i in range(10)]
-        for i, page in enumerate(pages): page.sequence = i
+        for i, page in enumerate(pages):
+            page.sequence = i
         data = BytesIO(b"".join([page.write() for page in pages]))
         self.failUnlessEqual(
             OggPage.find_last(data, pages[0].serial), pages[-1])
@@ -298,14 +305,16 @@ class TOggPage(TestCase):
     def test_find_last_really_last(self):
         pages = [OggPage() for i in range(10)]
         pages[-1].last = True
-        for i, page in enumerate(pages): page.sequence = i
+        for i, page in enumerate(pages):
+            page.sequence = i
         data = BytesIO(b"".join([page.write() for page in pages]))
         self.failUnlessEqual(
             OggPage.find_last(data, pages[0].serial), pages[-1])
 
     def test_find_last_muxed(self):
         pages = [OggPage() for i in range(10)]
-        for i, page in enumerate(pages): page.sequence = i
+        for i, page in enumerate(pages):
+            page.sequence = i
         pages[-2].last = True
         pages[-1].serial = pages[0].serial + 1
         data = BytesIO(b"".join([page.write() for page in pages]))
@@ -314,7 +323,8 @@ class TOggPage(TestCase):
 
     def test_find_last_no_serial(self):
         pages = [OggPage() for i in range(10)]
-        for i, page in enumerate(pages): page.sequence = i
+        for i, page in enumerate(pages):
+            page.sequence = i
         data = BytesIO(b"".join([page.write() for page in pages]))
         self.failUnless(OggPage.find_last(data, pages[0].serial + 1) is None)
 
@@ -325,9 +335,9 @@ class TOggPage(TestCase):
     # Disabled because GStreamer will write Oggs with bad data,
     # which we need to make a best guess for.
     #
-    #def test_find_last_invalid_sync(self):
-    #    data = BytesIO("if you think this is an OggS, you're crazy")
-    #    self.failUnlessRaises(OggError, OggPage.find_last, data, 0)
+    # def test_find_last_invalid_sync(self):
+    #     data = BytesIO("if you think this is an OggS, you're crazy")
+    #     self.failUnlessRaises(OggError, OggPage.find_last, data, 0)
 
     def test_find_last_invalid_sync(self):
         data = BytesIO(b"if you think this is an OggS, you're crazy")
@@ -342,8 +352,10 @@ class TOggPage(TestCase):
 
         import zlib
         old_crc = zlib.crc32
+
         def zlib_uint(*args):
             return (old_crc(*args) & 0xffffffff)
+
         def zlib_int(*args):
             return cdata.int_be(cdata.to_uint_be(old_crc(*args) & 0xffffffff))
 
@@ -353,7 +365,7 @@ class TOggPage(TestCase):
             zlib.crc32 = zlib_uint
             uint_data = page.write()
             zlib.crc32 = zlib_int
-            int_data  = page.write()
+            int_data = page.write()
         finally:
             zlib.crc32 = old_crc
 
@@ -361,9 +373,10 @@ class TOggPage(TestCase):
 
     def tearDown(self):
         self.fileobj.close()
-add(TOggPage)
 
-class TOggFileType(TestCase):
+
+class TOggFileTypeMixin(object):
+
     def scan_file(self):
         fileobj = open(self.filename, "rb")
         try:
@@ -434,20 +447,20 @@ class TOggFileType(TestCase):
         self.scan_file()
 
     def test_really_big(self):
-        self.audio["foo"] = "foo" * (2**16)
-        self.audio["bar"] = "bar" * (2**16)
-        self.audio["baz"] = "quux" * (2**16)
+        self.audio["foo"] = "foo" * (2 ** 16)
+        self.audio["bar"] = "bar" * (2 ** 16)
+        self.audio["baz"] = "quux" * (2 ** 16)
         self.audio.save()
         audio = self.Kind(self.filename)
-        self.failUnlessEqual(audio["foo"], ["foo" * 2**16])
-        self.failUnlessEqual(audio["bar"], ["bar" * 2**16])
-        self.failUnlessEqual(audio["baz"], ["quux" * 2**16])
+        self.failUnlessEqual(audio["foo"], ["foo" * 2 ** 16])
+        self.failUnlessEqual(audio["bar"], ["bar" * 2 ** 16])
+        self.failUnlessEqual(audio["baz"], ["quux" * 2 ** 16])
         self.scan_file()
 
     def test_delete_really_big(self):
-        self.audio["foo"] = "foo" * (2**16)
-        self.audio["bar"] = "bar" * (2**16)
-        self.audio["baz"] = "quux" * (2**16)
+        self.audio["foo"] = "foo" * (2 ** 16)
+        self.audio["bar"] = "bar" * (2 ** 16)
+        self.audio["baz"] = "quux" * (2 ** 16)
         self.audio.save()
 
         self.audio.delete()
@@ -457,30 +470,27 @@ class TOggFileType(TestCase):
 
     def test_invalid_open(self):
         self.failUnlessRaises(IOError, self.Kind,
-                              os.path.join('tests', 'data', 'xing.mp3'))
+                              os.path.join(DATA_DIR, 'xing.mp3'))
 
     def test_invalid_delete(self):
         self.failUnlessRaises(IOError, self.audio.delete,
-                              os.path.join('tests', 'data', 'xing.mp3'))
+                              os.path.join(DATA_DIR, 'xing.mp3'))
 
     def test_invalid_save(self):
         self.failUnlessRaises(IOError, self.audio.save,
-                              os.path.join('tests', 'data', 'xing.mp3'))
+                              os.path.join(DATA_DIR, 'xing.mp3'))
 
     def ogg_reference(self, filename):
         self.scan_file()
         if have_ogginfo:
-            value = os.system("ogginfo %s > %s 2> %s" % (filename, devnull,
-                              devnull))
-            self.failIf(value and value != NOTFOUND,
-                        "ogginfo failed on %s" % filename)
+            self.assertEqual(call_ogginfo(filename), 0,
+                             msg="ogginfo failed on %s" % filename)
+
         if have_oggz_validate:
             if filename.endswith(".opus") and not have_oggz_validate_opus:
                 return
-            value = os.system(
-                "oggz-validate %s > %s" % (filename, devnull))
-            self.failIf(value and value != NOTFOUND,
-                        "oggz-validate failed on %s" % filename)
+            self.assertEqual(call_oggz_validate(filename), 0,
+                             msg="oggz-validate failed on %s" % filename)
 
     def test_ogg_reference_simple_save(self):
         self.audio.save()
@@ -494,7 +504,7 @@ class TOggFileType(TestCase):
     def test_ogg_reference_delete(self):
         self.audio.delete()
         self.ogg_reference(self.filename)
- 
+
     def test_ogg_reference_medium_sized(self):
         self.audio["foobar"] = "foobar" * 1000
         self.audio.save()
@@ -509,29 +519,58 @@ class TOggFileType(TestCase):
 
     def test_mime_secondary(self):
         self.failUnless('application/ogg' in self.audio.mime)
- 
-    def tearDown(self):
-        os.unlink(self.filename)
 
-NOTFOUND = os.system("tools/notarealprogram 2> %s" % devnull)
+
+def call_ogginfo(*args):
+    with open(os.devnull, 'wb') as null:
+        return subprocess.call(
+            ["ogginfo"] + list(args), stdout=null, stderr=subprocess.STDOUT)
+
+
+def call_oggz_validate(*args):
+    with open(os.devnull, 'wb') as null:
+        return subprocess.call(
+            ["oggz-validate"] + list(args),
+            stdout=null, stderr=subprocess.STDOUT)
+
+
+def get_oggz_validate_version():
+    """A version tuple or OSError if oggz-validate isn't available"""
+
+    process = subprocess.Popen(["oggz-validate", "--version"],
+                               stdout=subprocess.PIPE)
+    output, unused_err = process.communicate()
+    retcode = process.poll()
+    if retcode != 0:
+        return (0,)
+    lines = output.splitlines()
+    if not lines:
+        return (0,)
+    parts = lines[0].split()
+    if not parts:
+        return (0,)
+    try:
+        return tuple(map(int, parts[-1].split(b".")))
+    except ValueError:
+        return (0,)
+
 
 have_ogginfo = True
-if os.system("ogginfo 2> %s > %s" % (devnull, devnull)) == NOTFOUND:
+try:
+    call_ogginfo()
+except OSError:
     have_ogginfo = False
     print("WARNING: Skipping ogginfo reference tests.")
+
+
 have_oggz_validate = True
 have_oggz_validate_opus = True
-if os.system("oggz-validate 2> %s > %s" % (devnull, devnull)) == NOTFOUND:
+try:
+    call_oggz_validate()
+except OSError:
     have_oggz_validate = False
     print("WARNING: Skipping oggz-validate reference tests.")
 else:
-    f = os.popen("oggz-validate --version")
-    try:
-        version_string = f.read()
-        version_part = version_string.split()[-1]
-        version = tuple(map(int, version_part.split(".")))
-        if version <= (0, 9, 9):
-            have_oggz_validate_opus = False
-            print("WARNING: Skipping oggz-validate reference tests for opus")
-    finally:
-        f.close()
+    if get_oggz_validate_version() <= (0, 9, 9):
+        have_oggz_validate_opus = False
+        print("WARNING: Skipping oggz-validate reference tests for opus")
