@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 import os
 import shutil
 from tempfile import mkstemp
@@ -5,20 +7,23 @@ from tempfile import mkstemp
 from mutagen._compat import BytesIO
 from mutagen.oggopus import OggOpus, OggOpusInfo, delete
 from mutagen.ogg import OggPage
-from tests import add
-from tests.test_ogg import TOggFileType
+from tests import TestCase, DATA_DIR
+from tests.test_ogg import TOggFileTypeMixin
 
 
-class TOggOpus(TOggFileType):
+class TOggOpus(TestCase, TOggFileTypeMixin):
     Kind = OggOpus
 
     def setUp(self):
-        original = os.path.join("tests", "data", "example.opus")
+        original = os.path.join(DATA_DIR, "example.opus")
         fd, self.filename = mkstemp(suffix='.opus')
         os.close(fd)
         shutil.copy(original, self.filename)
         self.audio = self.Kind(self.filename)
 
+    def tearDown(self):
+        os.unlink(self.filename)
+
     def test_length(self):
         self.failUnlessAlmostEqual(self.audio.info.length, 11.35, 2)
 
@@ -73,5 +78,3 @@ class TOggOpus(TOggFileType):
             page = OggPage(fobj)
             data = OggPage.to_packets([page])[0]
             self.assertTrue(data.endswith(b"\x01" + extra_data))
-
-add(TOggOpus)
