@@ -1,23 +1,29 @@
+# -*- coding: utf-8 -*-
+
 import os
 import shutil
 
 from mutagen._compat import cBytesIO
 from mutagen.ogg import OggPage
 from mutagen.oggvorbis import OggVorbis, OggVorbisInfo, delete
-from tests import add
-from tests.test_ogg import TOggFileType
+from tests import TestCase, DATA_DIR
+from tests.test_ogg import TOggFileTypeMixin
 from tempfile import mkstemp
 
-class TOggVorbis(TOggFileType):
+
+class TOggVorbis(TestCase, TOggFileTypeMixin):
     Kind = OggVorbis
-    
+
     def setUp(self):
-        original = os.path.join("tests", "data", "empty.ogg")
+        original = os.path.join(DATA_DIR, "empty.ogg")
         fd, self.filename = mkstemp(suffix='.ogg')
         os.close(fd)
         shutil.copy(original, self.filename)
         self.audio = self.Kind(self.filename)
 
+    def tearDown(self):
+        os.unlink(self.filename)
+
     def test_module_delete(self):
         delete(self.filename)
         self.scan_file()
@@ -81,21 +87,24 @@ class TOggVorbis(TOggFileType):
     def test_vorbiscomment(self):
         self.audio.save()
         self.scan_file()
-        if ogg is None: return
+        if ogg is None:
+            return
         self.failUnless(ogg.vorbis.VorbisFile(self.filename))
 
     def test_vorbiscomment_big(self):
         self.test_really_big()
         self.audio.save()
         self.scan_file()
-        if ogg is None: return
+        if ogg is None:
+            return
         vfc = ogg.vorbis.VorbisFile(self.filename).comment()
         self.failUnlessEqual(self.audio["foo"], vfc["foo"])
 
     def test_vorbiscomment_delete(self):
         self.audio.delete()
         self.scan_file()
-        if ogg is None: return
+        if ogg is None:
+            return
         vfc = ogg.vorbis.VorbisFile(self.filename).comment()
         self.failUnlessEqual(vfc.keys(), ["VENDOR"])
 
@@ -105,7 +114,8 @@ class TOggVorbis(TOggFileType):
         self.audio["foobar"] = "foobar" * 1000
         self.audio.save()
         self.scan_file()
-        if ogg is None: return
+        if ogg is None:
+            return
         vfc = ogg.vorbis.VorbisFile(self.filename).comment()
         self.failUnlessEqual(self.audio["foobar"], vfc["foobar"])
         self.failUnless("FOOBAR" in vfc.keys())
@@ -113,7 +123,7 @@ class TOggVorbis(TOggFileType):
 
     def test_huge_tag(self):
         vorbis = self.Kind(
-            os.path.join("tests", "data", "multipagecomment.ogg"))
+            os.path.join(DATA_DIR, "multipagecomment.ogg"))
         self.failUnless("big" in vorbis.tags)
         self.failUnless("bigger" in vorbis.tags)
         self.failUnlessEqual(vorbis.tags["big"], ["foobar" * 10000])
@@ -121,13 +131,13 @@ class TOggVorbis(TOggFileType):
         self.scan_file()
 
     def test_not_my_ogg(self):
-        fn = os.path.join('tests', 'data', 'empty.oggflac')
+        fn = os.path.join(DATA_DIR, 'empty.oggflac')
         self.failUnlessRaises(IOError, type(self.audio), fn)
         self.failUnlessRaises(IOError, self.audio.save, fn)
         self.failUnlessRaises(IOError, self.audio.delete, fn)
 
     def test_save_split_setup_packet(self):
-        fn = os.path.join("tests", "data", "multipage-setup.ogg")
+        fn = os.path.join(DATA_DIR, "multipage-setup.ogg")
         shutil.copy(fn, self.filename)
         audio = OggVorbis(self.filename)
         tags = audio.tags
@@ -137,7 +147,8 @@ class TOggVorbis(TOggFileType):
         self.failUnlessEqual(self.audio.tags, tags)
 
     def test_save_split_setup_packet_reference(self):
-        if ogg is None: return
+        if ogg is None:
+            return
         self.test_save_split_setup_packet()
         vfc = ogg.vorbis.VorbisFile(self.filename).comment()
         for key in self.audio:
@@ -145,8 +156,9 @@ class TOggVorbis(TOggFileType):
         self.ogg_reference(self.filename)
 
     def test_save_grown_split_setup_packet_reference(self):
-        if ogg is None: return
-        fn = os.path.join("tests", "data", "multipage-setup.ogg")
+        if ogg is None:
+            return
+        fn = os.path.join(DATA_DIR, "multipage-setup.ogg")
         shutil.copy(fn, self.filename)
         audio = OggVorbis(self.filename)
         audio["foobar"] = ["quux" * 50000]
@@ -163,9 +175,8 @@ class TOggVorbis(TOggFileType):
     def test_mime(self):
         self.failUnless("audio/vorbis" in self.audio.mime)
 
-try: import ogg.vorbis
+try:
+    import ogg.vorbis
 except ImportError:
     print("WARNING: Skipping Ogg Vorbis reference tests.")
     ogg = None
-
-add(TOggVorbis)
